<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class EventEditTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test **/
    public function can_edit_event()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $fieldA = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
            ]);

        factory(Response::class)->create(['field_id' => $fieldA->id, 'value' => 'Hello World!']);

        $fieldB = factory(Field::class)
            ->states('Select')
            ->create([
                'label' => 'Event type',
                'name' => 'event_type',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
                'data' => [
                    'options' => [
                        ['label' => 'Seminar', 'value' => 'seminar'],
                        ['label' => 'Single', 'value' => 'single'],
                    ],
                ],
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldA->id,
                'value' => 'Hello World!',
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldB->id,
                'value' => ['label' => 'Single', 'value' => 'single'],
            ]);

        $this->assertMatchesJsonSnapshot(
            $this->get(route('admin.event.edit', $event->id))->props()
        );
    }

    /** @test **/
    public function field_responses_are_audited_when_an_event_is_changed()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $fieldA = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
            ]);

        config(['audit.console' => true]); // This needs to be before create, or no update audit is created

        $fieldResponse = factory(Response::class)
            ->create([
                'field_id' => $fieldA->id,
                'value' => 'Original Name',
            ]);

        $this->withoutExceptionHandling()
            ->followingRedirects()
            ->post(route('admin.event.update', $event->id), [
                'name' => 'Updated Name',
            ])
            ->assertOk();

        $this->assertCount(2, $fieldResponse->audits); // Created and updated

        $updatedAudit = $fieldResponse->fresh()->audits->last();

        $this->assertArraySubset([
            'data' => '{"value":"Original Name"}',
        ], $updatedAudit->old_values);

        $this->assertArraySubset([
            'data' => '{"value":"Updated Name"}',
        ], $updatedAudit->new_values);
    }
}
